module DkComposer
  module Composefile
    module Service
      KEYWORDS = {
        image: :string,
        environment: :hash,
        ports: :array,
        links: :array,
        depends_on: :array,
        healthcheck: :hash,
        command: :string,
        extra_hosts: :array,
        volumes: :array,
        networks: :array,
        version: :string,
        stdin_open: :string,
        privileged: :string,
        hostname: :string,
        domainname: :string,
        container_name: :string,
        cap_add: :array,
        restart: :string,
        tty: :string
      }.freeze
      def method_missing(method, *params, **opts, &block)
        if KEYWORDS.key?(method)
          @fields ||= {}
          case KEYWORDS[method]
          when :array
            @fields[method] = params.flatten
          when :hash
            @fields[method] = if params.empty?
                                opts.stringfy_bool_value
                              else
                                params.map { |x| x.split('=') }.to_h.stringfy_bool_value
                              end
          else
            v, = params
            @fields[method] = v
          end
        else
          super
        end
      end
    end

    module Volume
      KEYWORDS = {
        driver: :string,
        driver_opts: :hash,
        external: :string,
        lables: :hash
      }.freeze
      def method_missing(method, *params, **opts, &block)
        if KEYWORDS.key?(method)
          @fields ||= {}
          case KEYWORDS[method]
          when :array
            @fields[method] = params
          when :hash
            @fields[method] = if params.empty?
                                opts.stringfy_bool_value
                              else
                                params.map { |x| x.split('=') }.to_h.stringfy_bool_value
                              end
          else
            v, = params
            @fields[method] = v
          end
        else
          super
        end
      end
    end

    module Network
      KEYWORDS = {
        driver: :string,
        driver_opts: :hash,
        attachable: :string,
        enable_ipv6: :string,
        ipam: :hash,
        internal: :string,
        external: :hash,
        lables: :hash
      }.freeze
      def method_missing(method, *params, **opts, &block)
        if KEYWORDS.key?(method)
          @fields ||= {}
          case KEYWORDS[method]
          when :array
            @fields[method] = params
          when :hash
            @fields[method] = if params.empty?
                                opts.stringfy_bool_value
                              else
                                params.map { |x| x.split('=') }.to_h.stringfy_bool_value
                              end
          else
            v, = params
            @fields[method] = v
          end
        else
          super
        end
      end
    end
  end
end
