module DkComposer
  class Image
    include Dockerfile
    include Helper
    attr_accessor :from, :actions, :name, :repos, :tag
    def initialize(name)
      @name = name
      @actions = []
      @repos = []
      @tag = 'latest'
    end

    def from(param = nil)
      return @from unless param
      @from = if param.is_a? Symbol
                puts "WARNING: There is no image definition named: :#{param}! Pls require the def file " unless IMAGE.key?(param)
                IMAGE[param].repo
              else
                param
              end
    end

    def repo
      repos.first
    end

    def repos(*values)
      if values.empty?
        return ["#{name}:#{tag || 'latest'}"] if @repos.empty?
        @repos
      else
        @repos.concat values
      end
    end

    def tag(values = nil)
      if values
        @tag = values

      else
        @tag || 'latest'
      end
    end

    def use(*params)
      return @use if params.empty?
      @use = params
      params.each do |image_name|
        puts "WARNING: There is no image definition named: :#{image_name}! Pls require the def file " unless IMAGE.key?(image_name)

        @actions << ['', "#>>>>>>>>>>>>>#{image_name} begin<<<<<<<<<<<<<<<<<<<"]
        @actions.concat IMAGE[image_name].actions
        @actions << ['', "#>>>>>>>>>>>>>#{image_name} end<<<<<<<<<<<<<<<<<<<<<"]
        @actions << ['', '                                                   ']
      end
    end

    def pull
      repos.each do |repo|
        pull_cmd = "docker pull #{repo}"
        run_command(pull_cmd)
      end
    end

    def push
      repos.each do |repo|
        push_cmd = "docker push #{repo}"
        run_command(push_cmd)
      end
    end

    def build(path = build_path)
      dockerfile = tempfile(to_docker_file, 'Dockerfile', path)
      build_cmd = [
        "pushd #{path}",
        "docker build -f #{dockerfile}  #{repos.map { |repo| '-t ' + repo }.join(' ')} .",
        'popd'
      ].join('&&')
      run_command(build_cmd)
    end

    def to_docker_file
      return '' if actions.empty? && @from.nil?
      actions.clone.unshift([:from, @from]).map do |instruct, *params|
        "#{instruct.to_s.upcase} #{params.join(' ')}"
      end.join("\n")
    end

    def to_s
      ret = <<-DUMP
     image(:#{@name}) do
        from "#{@from}"
        repos #{@repos.map { |r| "'#{r}'" }.join(',')}
        tag :#{@tag}
        #{actions.map(&:flatten).map { |a| a.join(' ') }.join("\n")}
     end
    DUMP
      ret
    end

    def full_name
      "#{@name}:#{tag}"
    end
    alias dump to_s
    def self.create(name, *params, **opts, &block)
      if name.is_a?(String)
        opts[:repos] ||= []
        opts[:repos].unshift(name)
      end

      ret = Image.new(name)
      ret.config(opts)
      ret.instance_exec(*params, &block) if block
      ret
    end
  end
end

def image(name, *params, **opts, &block)
  ret = DkComposer::Image.create(name, *params, **opts, &block)
  shortdesc, longdesc = (@desc || []).pop
  ret.shortdesc = shortdesc
  ret.longdesc = longdesc
  DkComposer::IMAGE[ret.full_name.to_sym] = ret
  DkComposer::IMAGE[name] = DkComposer::IMAGE["#{name}:latest".to_sym]
  ret
end
